##
#   CBpanel - Panel Application
#
#   by ADcomp <david.madbox@gmail.com>
#      http://www.ad-comp.be/
#
#   | Based on Barpanel
#   | Copyright (C) 2005 - 2006, Federico Pelloni <federico.pelloni@gmail.com>
#   | Copyright (C) 2008 Job and Esther Technologies Ltd
#
#   This program is distributed under the terms of the GNU General Public License
#   For more info see http://www.gnu.org/licenses/gpl.txt
##

import os
import os.path
import locale
import sys

import gtk
import pango

import datetime, traceback, shutil

themepaths = [ os.path.realpath(os.path.join(os.path.dirname(__file__), "themes")),
               os.path.expanduser("~/.config/cbpanel/themes") ]

#~ tooltips = gtk.Tooltips()

def log(level, message):
    pass
    sys.stderr.write("[%-7s] %s\n" % (level, message.replace("\n", " ")))
    sys.stderr.flush()

def logINFO(message):
    log("INFO", message)

def logERROR(message):
    log("ERROR", message)

def logWARNING(message):
    log("WARNING", message)

def logDEBUG(message):
    log("DEBUG", message)

class PluginManager:
    def __init__( self, panel ):
        if panel.paneltype == "top" or panel.paneltype == "bottom":
            self.box = gtk.HBox(False, 0)
        else:
            self.box = gtk.VBox(False, 0)
        self.box.set_border_width(0)
        self.box.set_spacing(0)
        self.panel = panel
        self.plugins = []
        #~ panel.frame_box.add(self.box)
        panel.add(self.box)
        self.box.set_name( "CBpanelPluginManager" )
        self.box.show_all()

    def load_plugin(self, p, settings=None, position = -1):
        plugwidget = None
        try:
            logINFO(("Loading plugin '%s' ..") % (p))
            #if p == 'menu':
            #    exec("import CBpanel.plugins.%s.%s as plugin" % (p, self.panel.config['MENUTYPE']))
            #else:
            #    exec("import CBpanel.plugins.%s as plugin" % p)
            exec("import CBpanel.plugins.%s as plugin" % p)
            if settings == None:
                settings = {}
            settings['pluginname'] = plugin.name
            plugwidget = plugin.Plugin(self.panel, settings)
            expand = False
            if plugin.name == 'WindowList':
                expand = True
            self.box.pack_start(plugwidget, expand, True)
            self.plugins.append(plugwidget)
            plugwidget.show()
        except Exception, e:
            logWARNING(("Unable to load plugin '%s': %s") % (p, e))
            traceback.print_exc()
        return plugwidget

class Plugin(gtk.HBox):
    def __init__(self, panel, settings):
        gtk.HBox.__init__(self, False, 0)
        self.set_border_width(0)
        self.set_spacing(0)

        self.settings = settings
        if settings.has_key('pluginname'):
            self.pluginname = settings['pluginname']
        if self.pluginname==None or len(self.pluginname)<1:
            self.pluginname = "CBpanel.Plugin"
        self.panel = panel
        if self.settings.has_key("size"):
            if self.panel.paneltype == "bottom" or self.panel.paneltype == "top":
                self.set_size_request(int(self.settings['size']), -1)
            else:
                self.set_size_request(-1, int(self.settings['size']))
        self.show()

    def logINFO(self, message):
        logINFO("%s: %s" % (self.pluginname, message))
    def logERROR(self, message):
        logERROR("%s: %s" % (self.pluginname, message))
    def logWARNING(self, message):
        logWARNING("%s: %s" % (self.pluginname, message))
    def logDEBUG(self, message):
        logDEBUG("%s: %s" % (self.pluginname, message))

    def resize(self):
        pass

class panelWindow(gtk.Window):
    def __init__(self, wtype = gtk.WINDOW_TOPLEVEL):
        gtk.Window.__init__(self, wtype)
        self.set_decorated(False)
        self.set_skip_taskbar_hint(True)
        self.set_skip_pager_hint(True)
        self.set_border_width(0)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
        #~ self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#0000FF"))

#-----------------------------------------------------------------------
class Panel(panelWindow):
#-----------------------------------------------------------------------
    def __init__(self, config, launch_list):
        panelWindow.__init__( self )
        self.add_events(gtk.gdk.BUTTON_PRESS_MASK|
                               gtk.gdk.ENTER_NOTIFY|
                               gtk.gdk.LEAVE_NOTIFY)
        self.stick()
        #~ self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DESKTOP)

        ## Events
        self.connect("enter-notify-event", self.onMouseEnter)
        self.connect("leave-notify-event", self.onMouseLeave)
        self.connect("configure-event", self.onConfigureEvent)
        self.connect("destroy", self.doquit)

        self.config = config
        self.align = config['ALIGN']
        self.paneltype = config['PANELTYPE']
        self.span = config['SPAN']
        self.autohide = config['AUTOHIDE']
        if self.span == None or len(self.span)<1:
            self.span = "100%"
        self.size = locale.atoi(config['SIZE'])
        self.iconsize = locale.atoi(config['ICONSIZE'])
        self.fontdesc = config['FONT']
        self.launch_list = launch_list

        if self.autohide == "yes":
            self.set_keep_above(True)
        else:
            self.set_keep_below(True)

        ## Config
        self.opened_popup = None
        self.set_title("CBpanel")
        self.set_name("CBpanelWindow")

        #~ self.frame_box = gtk.Frame()
        #~ self.frame_box.set_border_width(0)
        #~ self.frame_box.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        # gtk.SHADOW_NONE, gtk.SHADOW_IN, gtk.SHADOW_OUT, gtk.SHADOW_ETCHED_IN, gtk.SHADOW_ETCHED_OUT
        #~ self.frame_box.show()
        #~ self.add(self.frame_box)

        self.lastw = 0
        self.lasth = 0
        self.hasmouse = False
        self.plugin_mgr = PluginManager(self)

        self.root_path = None


    def onMouseEnter(self, widget, event):
        if self.hasmouse == False:
            self.hasmouse = True
            self.displayPanel()

    def onMouseLeave(self, widget, event):
        if self.hasmouse == True and event.detail != gtk.gdk.NOTIFY_INFERIOR: # 2
            self.hasmouse = False
            self.displayPanel()

    def displayPanel(self):
        self.resize()
        self.show_all()

    def resize(self):
        if self.paneltype == "bottom" or self.paneltype == "top":
            w = gtk.gdk.screen_width()
            if self.span.find("%") > 0:
                nw = locale.atoi(self.span.replace("%", "")) * w / 100
            else:
                nw = locale.atoi(self.span)
            if nw < w:
                w = nw
            size = self.size
            if self.autohide == "yes" and self.hasmouse == False and self.opened_popup == None:
                size = 1
            panelWindow.resize(self, w, size)
            self.set_size_request(w, size)
        elif self.paneltype == "right" or self.paneltype == "left":
            h = self.available_height()
            if self.span.find("%") > 0:
                nh = locale.atoi(self.span.replace("%", "")) * h / 100
            else:
                nh = locale.atoi(self.span)
            if nh < h:
                h = nh
            size = self.size
            if self.autohide == "yes" and self.hasmouse == False and self.opened_popup == None:
                size = 1
            cb_panelWindow.resize(self, size, h)
            self.set_size_request(size, h)
        self.reposition()

    def reposition( self ):
        screen_width = gtk.gdk.screen_width()
        screen_height = gtk.gdk.screen_height()
        panel_width, panel_height = self.get_size()

        if self.paneltype == "top":
            x = 0
            y = 0
        elif self.paneltype == "bottom":
            x = 0
            y = screen_height  - panel_height

        if self.align == "left":
            x = 0
        elif self.align == "right":
            x =  screen_width - panel_width
        elif self.align == "center":
            x =  (screen_width - panel_width) / 2


        self.move( x, y )
        if self.opened_popup!=None:
            self.opened_popup.reposition()
        self.strut_set()

    def strut_set(self):
        if self.window:
            w, h = self.get_size()
            x, y = self.get_position()
            if self.autohide == "yes":
                if self.paneltype == "bottom" or self.paneltype == "top":
                    h = 1
                else:
                    w = 1
            if self.paneltype == "bottom":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [0,0,0,h])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [0,0,0,h, 0,0,0,0, 0,0,0,w])
            elif self.paneltype == "top":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [0,0,h,0])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [0,0,h,0, 0,0,0,0, 0,w,0,0])
            elif self.paneltype == "left":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [w,0,0,0])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [w,0,0,0, 0,h,0,0, 0,0,0,0])
            elif self.paneltype == "right":
                self.window.property_change("_NET_WM_STRUT", "CARDINAL", 32, gtk.gdk.PROP_MODE_REPLACE, [0,w,0,0])
                self.window.property_change("_NET_WM_STRUT_PARTIAL", "CARDINAL", \
                                                32, gtk.gdk.PROP_MODE_REPLACE, [0,w,0,0, 0,0,0,h, 0,0,0,0])

    def update_font(self, plugin, child):
        if self.fontdesc!=None and len(self.fontdesc)>0:
            child.modify_font(pango.FontDescription(self.fontdesc))

    def get_image_from_icon_name(self, icon, pixelsize=-1):
        img = gtk.image_new_from_icon_name(icon, gtk.ICON_SIZE_BUTTON)
        if pixelsize < 0:
            img.set_pixel_size(self.iconsize)
        return(img)

    def get_stock_image(self, stockid, pixelsize=-1):
        img = gtk.image_new_from_stock(stockid, gtk.ICON_SIZE_BUTTON)
        if pixelsize < 0:
            img.set_pixel_size(self.iconsize)
        return(img)

    def update_gtk_image(self, image, file, pixelsize=-1):
        if file[0] == '/':
            path = file
        else:
            path = "%s/images/%s" % (os.path.realpath(os.path.dirname(__file__)), file)
            print "%s/images/%s" % (os.path.realpath(os.path.dirname(__file__)), file)
        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file(path)
        except:
            print "Error loading image .."

        if pixelsize < 0:
            pixelsize = self.iconsize
        bw = pixbuf.get_width()
        bh = pixbuf.get_height()
        if bh!=pixelsize: # or bw!=pixelsize
            nw = pixelsize * bw / bh
            scaled_buf = pixbuf.scale_simple(nw,pixelsize,gtk.gdk.INTERP_BILINEAR)
            image.set_from_pixbuf(scaled_buf)
        else:
            image.set_from_pixbuf(pixbuf)

    def get_image_from_file(self, file, pixelsize=-1):
        v = gtk.Image()
        try:
            self.update_gtk_image(v, file, pixelsize)
        except:
            print ' .. bug .. '
            v = self.get_image_from_icon_name('image-missing')
        return(v)

    def onConfigureEvent(self, window, event):
        if event.width != self.lastw or event.height != self.lasth:
            self.lastw = event.width
            self.lasth = event.height
            self.reposition()
            self.strut_set()


    def doquit(self, widget=None, data=None):
        """ Quit Main loop """
        for plugin in self.plugin_mgr.plugins:
            if plugin.pluginname == 'Systray':
                plugin.destroy()

        gtk.main_quit()

    def run(self):
        """ Main loop """
        self.displayPanel()
        gtk.main()


class PopupWindow(panelWindow):
    def __init__(self, panel, border = 3, wtype = gtk.WINDOW_POPUP):
        panelWindow.__init__(self, wtype)
        self.focus_check = False
        self.panel = panel
        self.set_name("CBpanelPopupWindow")
        self.set_border_width(border)
        self.add_events(gtk.gdk.FOCUS_CHANGE_MASK)
        self.connect("focus-out-event", self.lost_focus)
        self.opener = None
        self._box = gtk.VBox()
        self._box.show()
        self._box.set_spacing(3)
        gtk.Window.add(self, self._box)

    def add(self, widget, expand = False):
        self._box.pack_start(widget, expand, expand)

    def set_header(self, title = None, description = None, icon = None):
        frame = gtk.Frame()
        frame.show()
        self.add(frame)
        self.header = gtk.HBox(False, 10)
        self.header.set_name("CBpanelPopupHeader")
        self.header.show()
        self.header.set_border_width(3)
        frame.add(self.header)
        if icon:
            if type(icon) == gtk.gdk.Pixbuf:
                image = gtk.Image()
                image.set_from_pixbuf(icon)
            elif type(icon) == gtk.Image:
                image = icon
            image.show()
            self.header.pack_start(image, False, False)
        if title or description:
            lbl = gtk.Label("")
            txt = "<big>"
            if title:
                txt += "<b>%s</b>" % title
                if description:
                    txt += "\n"
            if description:
                txt += description
            txt += "</big>"
            lbl.set_markup(txt)
            lbl.set_property("xalign", 0)
            lbl.show()
            self.header.add(lbl)

    def run(self):
        self.show()

    def lost_focus(self, widget, event):
        if self.focus_check:
            self.toggle()

    def reposition(self):
        window_x, window_y = self.panel.get_position()
        window_w, window_h = self.panel.get_size()
        x = 0
        y = 0
        if self.panel.paneltype == "top" or self.panel.paneltype == "bottom":
            x = self.x + window_x
            h = self.get_size()[1]
            if self.panel.paneltype == "top":
                y = window_y + window_h
            elif self.panel.paneltype == "bottom":
                y = window_y - h
            myw = self.get_size()[0]
            while x >= 0 and x + myw > gtk.gdk.screen_width():
                x = x - 1
        else:
            y = self.y + window_y
            w = self.get_size()[0]
            if self.panel.paneltype == "left":
                x = window_x + window_w
            elif self.panel.paneltype == "right":
                x = window_x - w
            myh = self.get_size()[1]
            while y >= 0 and y + myh > gtk.gdk.screen_height():
                y = y - 1
        self.move(x, y)

    def open(self, widget = 0):
        if self.panel.opened_popup:
            if id(self) == id(self.panel.opened_popup):
                return
            self.panel.opened_popup.close()
        if type(widget) == type(1):
            self.x = widget
            self.y = widget
        else:
            self.opener = widget
            try:
                self.x = widget.allocation.x
                self.y = widget.allocation.y
            except:
                self.x = 0
                self.y = 0
                self.opener = None
        self.reposition()
        self.show()
        self.panel.opened_popup = self
        if self.panel.autohide == "yes":
            self.panel.resize()

    def close(self):
        self.hide()
        try:
            self.opener.set_active(False)
        except:
            raise
        self.panel.opened_popup = None
        if self.panel.autohide == "yes":
            self.panel.resize()

    def toggle(self, widget = 0):
        if self.panel.opened_popup:
            if id(self) != id(self.panel.opened_popup):
                self.panel.opened_popup.close()
                self.open(widget)
                self.focus_check = True
                ## need this to give focus to window !
                self.present()
            else:
                self.focus_check = False
                self.close()
        else:
            self.open(widget)
            self.focus_check = True
            ## need this to give focus to window !
            self.present()

    def toggleConditional(self, a, b):
        pass

class panelMenuWindow(PopupWindow):
    __is_size_set = False
    def __init__(self, name, description, icon, panel, border = 3, wtype = gtk.WINDOW_POPUP):
        PopupWindow.__init__(self, panel, border, wtype)
        #~ self.set_header(name, description, icon)
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_NONE)
        self.scroll.show()
        self.add(self.scroll, True)
        self.buttons = gtk.VBox(True, 0)
        self.buttons.show()
        self.scroll.add_with_viewport(self.buttons)

    def append(self, obj):
        self.buttons.add(obj)
        obj.show()

    def toggle(self, widget = 0):
        if not self.__is_size_set:
            w, h = self.buttons.size_request()
            pw, ph = self.panel.get_size()
            self.scroll.set_size_request(w, min(h + 5, gtk.gdk.screen_height()*8/10)) # - int(ph) - 200))
            self.__is_size_set = True
        self.scroll.get_vadjustment().set_value(0)
        PopupWindow.toggle(self, widget)

def load_theme(theme):
    print 'load theme %s' % theme
    try:
        logINFO("Loading theme '%s' .." % theme)
        path = None
        if theme == "User Theme":
            try:
                path = os.path.join(os.path.expanduser("~/.config/cbpanel/themes"), "gtkrc")
                if not os.path.isfile(path):
                    path = None
            except:
                path = None
        else:
            for p in themepaths:
                try:
                    path = os.path.join(p, theme, "gtkrc")
                    if os.path.isfile(path):
                        break
                    else:
                        path = None
                except:
                    path = None
            if path is None or not os.path.isfile(path):
                raise
        gtk.rc_reset_styles(gtk.settings_get_default())
        gtk.rc_parse(path)
    except:
        logINFO("Warning: unable to load theme %s, using default" % theme)
